@@ -1,7 +1,7 @@
 # Human friendly input/output in Python.
 #
 # Author: Peter Odding <peter@peterodding.com>
-# Last Change: October 9, 2016
+# Last Change: May 18, 2017
 # URL: https://humanfriendly.readthedocs.io
 
 """
@@ -17,6 +17,7 @@ escape sequences work.
 """
 
 # Standard library modules.
+import codecs
 import os
 import re
 import subprocess
@@ -36,7 +37,7 @@ except ImportError:
 # Modules included in our package. We import find_meta_variables() here to
 # preserve backwards compatibility with older versions of humanfriendly where
 # that function was defined in this module.
-from humanfriendly.compat import is_unicode
+from humanfriendly.compat import coerce_string, is_unicode
 from humanfriendly.text import concatenate, format
 from humanfriendly.usage import find_meta_variables, format_usage  # NOQA
 
@@ -82,6 +83,9 @@ DEFAULT_LINES = 25
 DEFAULT_COLUMNS = 80
 """The default number of columns in a terminal (an integer)."""
 
+DEFAULT_ENCODING = 'UTF-8'
+"""The output encoding for Unicode strings."""
+
 HIGHLIGHT_COLOR = os.environ.get('HUMANFRIENDLY_HIGHLIGHT_COLOR', 'green')
 """
 The color used to highlight important tokens in formatted text (e.g. the usage
@@ -91,36 +95,78 @@ message of the ``humanfriendly`` program). If the environment variable
 """
 
 
-def message(*args, **kw):
+def output(text, *args, **kw):
     """
-    Show an informational message on the terminal.
+    Print a formatted message to the standard output stream.
 
-    :param args: Any position arguments are passed on to :func:`~humanfriendly.text.format()`.
-    :param kw: Any keyword arguments are passed on to :func:`~humanfriendly.text.format()`.
+    For details about argument handling please refer to
+    :func:`~humanfriendly.text.format()`.
 
     Renders the message using :func:`~humanfriendly.text.format()` and writes
-    the resulting string to :data:`sys.stderr` (followed by a newline).
+    the resulting string (followed by a newline) to :data:`sys.stdout` using
+    :func:`auto_encode()`.
     """
-    sys.stderr.write(format(*args, **kw) + '\n')
+    auto_encode(sys.stdout, coerce_string(text) + '\n', *args, **kw)
 
 
-def warning(*args, **kw):
+def message(text, *args, **kw):
+    """
+    Print a formatted message to the standard error stream.
+
+    For details about argument handling please refer to
+    :func:`~humanfriendly.text.format()`.
+
+    Renders the message using :func:`~humanfriendly.text.format()` and writes
+    the resulting string (followed by a newline) to :data:`sys.stderr` using
+    :func:`auto_encode()`.
+    """
+    auto_encode(sys.stderr, coerce_string(text) + '\n', *args, **kw)
+
+
+def warning(text, *args, **kw):
     """
     Show a warning message on the terminal.
 
-    :param args: Any position arguments are passed on to :func:`~humanfriendly.text.format()`.
-    :param kw: Any keyword arguments are passed on to :func:`~humanfriendly.text.format()`.
+    For details about argument handling please refer to
+    :func:`~humanfriendly.text.format()`.
 
     Renders the message using :func:`~humanfriendly.text.format()` and writes
-    the resulting string to :data:`sys.stderr` (followed by a newline). If
-    :data:`sys.stderr` is connected to a terminal :func:`ansi_wrap()` is used
-    to color the message in a red font (to make the warning stand out from
-    surrounding text).
+    the resulting string (followed by a newline) to :data:`sys.stderr` using
+    :func:`auto_encode()`.
+
+    If :data:`sys.stderr` is connected to a terminal that supports colors,
+    :func:`ansi_wrap()` is used to color the message in a red font (to make
+    the warning stand out from surrounding text).
     """
-    text = format(*args, **kw)
+    text = coerce_string(text)
     if terminal_supports_colors(sys.stderr):
         text = ansi_wrap(text, color='red')
-    sys.stderr.write(text + '\n')
+    auto_encode(sys.stderr, text + '\n', *args, **kw)
+
+
+def auto_encode(stream, text, *args, **kw):
+    """
+    Reliably write Unicode strings to the terminal.
+
+    :param stream: The file-like object to write to (a value like
+                   :data:`sys.stdout` or :data:`sys.stderr`).
+    :param text: The text to write to the stream (a string).
+    :param args: Refer to :func:`~humanfriendly.text.format()`.
+    :param kw: Refer to :func:`~humanfriendly.text.format()`.
+
+    Renders the text using :func:`~humanfriendly.text.format()` and writes it
+    to the given stream. If an :exc:`~exceptions.UnicodeEncodeError` is
+    encountered in doing so, the text is encoded using :data:`DEFAULT_ENCODING`
+    and the write is retried. The reasoning behind this rather blunt approach
+    is that it's preferable to get output on the command line in the wrong
+    encoding then to have the Python program blow up with a
+    :exc:`~exceptions.UnicodeEncodeError` exception.
+    """
+    text = format(text, *args, **kw)
+    try:
+        stream.write(text)
+    except UnicodeEncodeError:
+        stream.write(codecs.encode(text, DEFAULT_ENCODING))
 
 
 def ansi_strip(text, readline_hints=True):
@@ -157,7 +203,7 @@ def ansi_style(**kw):
                enabled.
     :returns: The ANSI escape sequences to enable the requested text styles or
               an empty string if no styles were requested.
-    :raises: :py:exc:`~exceptions.ValueError` when an invalid color name is given.
+    :raises: :exc:`~exceptions.ValueError` when an invalid color name is given.
     """
     # Start with sequences that change text styles.
     sequences = [str(ANSI_TEXT_STYLES[k]) for k, v in kw.items() if k in ANSI_TEXT_STYLES and v]
@@ -372,7 +418,7 @@ def find_terminal_size():
               cached values!
 
               Response: As a library I don't consider it the role of the
-              :py:mod:`humanfriendly.terminal` module to install a process wide
+              :mod:`humanfriendly.terminal` module to install a process wide
               signal handler ...
 
     .. _Pre-emptive snarky comment: http://blogs.msdn.com/b/oldnewthing/archive/2008/01/30/7315957.aspx
@@ -452,33 +498,71 @@ def usage(usage_text):
     show_pager(usage_text)
 
 
-def show_pager(formatted_text, encoding='UTF-8'):
+def show_pager(formatted_text, encoding=DEFAULT_ENCODING):
     """
     Print a large text to the terminal using a pager.
 
     :param formatted_text: The text to print to the terminal (a string).
     :param encoding: The name of the text encoding used to encode the formatted
-                     text if the formatted text is a Unicode string (a string).
+                     text if the formatted text is a Unicode string (a string,
+                     defaults to :data:`DEFAULT_ENCODING`).
+
+    When :func:`connected_to_terminal()` returns :data:`True` a pager is used
+    to show the text on the terminal, otherwise the text is printed directly
+    without invoking a pager.
 
     The use of a pager helps to avoid the wall of text effect where the user
     has to scroll up to see where the output began (not very user friendly).
 
-    If :data:`sys.stdout` is not connected to a terminal (see
-    :func:`connected_to_terminal()`) then the text is printed directly without
-    invoking a pager.
-
-    If the given text contains ANSI escape sequences the command ``less
-    --RAW-CONTROL-CHARS`` is used, otherwise ``$PAGER`` is used (if ``$PAGER``
-    isn't set the command ``less`` is used).
+    Refer to :func:`get_pager_command()` for details about the command line
+    that's used to invoke the pager.
     """
-    if connected_to_terminal(sys.stdout):
-        if ANSI_CSI in formatted_text:
-            pager_command = ['less', '--RAW-CONTROL-CHARS']
-        else:
-            pager_command = [os.environ.get('PAGER', 'less')]
+    if connected_to_terminal():
+        command_line = get_pager_command(formatted_text)
+        pager = subprocess.Popen(command_line, stdin=subprocess.PIPE)
         if is_unicode(formatted_text):
             formatted_text = formatted_text.encode(encoding)
-        pager = subprocess.Popen(pager_command, stdin=subprocess.PIPE)
         pager.communicate(input=formatted_text)
     else:
-        print(formatted_text)
+        output(formatted_text)
+
+
+def get_pager_command(text=None):
+    """
+    Get the command to show a text on the terminal using a pager.
+
+    :param text: The text to print to the terminal (a string).
+    :returns: A list of strings with the pager command and arguments.
+
+    The use of a pager helps to avoid the wall of text effect where the user
+    has to scroll up to see where the output began (not very user friendly).
+
+    If the given text contains ANSI escape sequences the command ``less
+    --RAW-CONTROL-CHARS`` is used, otherwise the environment variable
+    ``$PAGER`` is used (if ``$PAGER`` isn't set less_ is used).
+
+    When the selected pager is less_, the following options are used to make
+    the experience more user friendly:
+
+    - ``--quit-if-one-screen`` causes less_ to automatically exit if the entire
+      text can be displayed on the first screen. This makes the use of a pager
+      transparent for smaller texts (because the operator doesn't have to quit
+      the pager).
+
+    - ``--no-init`` prevents less_ from clearing the screen when it exits. This
+      ensures that the operator gets a chance to review the text (for example a
+      usage message) after quitting the pager, while composing the next command.
+
+    .. _less: http://man7.org/linux/man-pages/man1/less.1.html
+    """
+    # Compose the pager command.
+    if text and ANSI_CSI in text:
+        command_line = ['less', '--RAW-CONTROL-CHARS']
+    else:
+        command_line = [os.environ.get('PAGER', 'less')]
+    # Pass some additional options to `less' (to make it more
+    # user friendly) without breaking support for other pagers.
+    if os.path.basename(command_line[0]) == 'less':
+        command_line.append('--no-init')
+        command_line.append('--quit-if-one-screen')
+    return command_line
